import { PostHog } from '../posthog-core';
import { Survey } from '../posthog-surveys-types';
export declare class SurveyManager {
    private posthog;
    private surveyInFocus;
    constructor(posthog: PostHog);
    private canShowNextEventBasedSurvey;
    private handlePopoverSurvey;
    private handleWidget;
    private handleWidgetSelector;
    /**
     * Sorts surveys by their appearance delay in ascending order. If a survey does not have an appearance delay,
     * it is considered to have a delay of 0.
     * @param surveys
     * @returns The surveys sorted by their appearance delay
     */
    private sortSurveysByAppearanceDelay;
    callSurveysAndEvaluateDisplayLogic: (forceReload?: boolean) => void;
    private addSurveyToFocus;
    private removeSurveyFromFocus;
    getTestAPI(): {
        addSurveyToFocus: (id: string) => void;
        removeSurveyFromFocus: (id: string) => void;
        surveyInFocus: string | null;
        canShowNextEventBasedSurvey: () => boolean;
        handleWidget: (survey: Survey) => void;
        handlePopoverSurvey: (survey: Survey) => void;
        handleWidgetSelector: (survey: Survey) => void;
        sortSurveysByAppearanceDelay: (surveys: Survey[]) => Survey[];
    };
}
export declare const renderSurveysPreview: ({ survey, parentElement, previewPageIndex, forceDisableHtml, }: {
    survey: Survey;
    parentElement: HTMLElement;
    previewPageIndex: number;
    forceDisableHtml?: boolean | undefined;
}) => void;
export declare const renderFeedbackWidgetPreview: ({ survey, root, forceDisableHtml, }: {
    survey: Survey;
    root: HTMLElement;
    forceDisableHtml?: boolean | undefined;
}) => void;
export declare function generateSurveys(posthog: PostHog): void;
export declare function usePopupVisibility(survey: Survey, posthog: PostHog | undefined, millisecondDelay: number, isPreviewMode: boolean, removeSurveyFromFocus: (id: string) => void): {
    isPopupVisible: boolean;
    isSurveySent: boolean;
    setIsPopupVisible: import("preact/hooks").StateUpdater<boolean>;
};
export declare function SurveyPopup({ survey, forceDisableHtml, posthog, style, previewPageIndex, removeSurveyFromFocus, }: {
    survey: Survey;
    forceDisableHtml?: boolean;
    posthog?: PostHog;
    style?: React.CSSProperties;
    previewPageIndex?: number | undefined;
    removeSurveyFromFocus: (id: string) => void;
}): JSX.Element;
export declare function Questions({ survey, forceDisableHtml, posthog, styleOverrides, }: {
    survey: Survey;
    forceDisableHtml: boolean;
    posthog?: PostHog;
    styleOverrides?: React.CSSProperties;
}): JSX.Element;
export declare function FeedbackWidget({ survey, forceDisableHtml, posthog, readOnly, removeSurveyFromFocus, }: {
    survey: Survey;
    forceDisableHtml?: boolean;
    posthog?: PostHog;
    readOnly?: boolean;
    removeSurveyFromFocus: (id: string) => void;
}): JSX.Element;
